{ File: grafvis2.pas }

{ Scopo: procedure di visita
         - iterativa in ampiezza e
         - ricorsiva in profonditadi un grafo

  Versioni ottimizzate dal punto di vista dell'occupazione di spazio. }

{ La procedura di visita in ampiezza richiede la definizione di code i cui
  elementi sono i nodi del grafo. }

type
  TipoElemCoda = TipoNodo;

{$I CODE.PAS}  { Inclusione della realizzazione del tipo di dato coda }

{ Le procedure di visita richiedono la definizione della procedura che analizza
  un nodo.  L'analisi di un nodo puo` essere ad esempio la sua stampa:
}

  procedure Analizza (i: TipoNodo);
  begin
    write(' ',i,' ')
  end;

{
  algoritmo visita in ampiezza un grafo a partire dal nodo n
  1.  togli la marca a tutti i nodi del grafo
  2.  coda dei nodi da visitare := coda vuota
  3.  marca n
  4.  metti n nella coda dei nodi da visitare
  5.  finche` la coda dei nodi da visitare non e' vuota
      ripeti
  6.      estrai un nodo i dalla coda dei nodi da visitare
  7.      analizza i (stampa il valore intero i)
  8.          per ogni successore j di i che non e' marcato
              ripeti
  9.              marca j
 10.              metti j nella coda dei nodi da visitare

  NB: Questo algoritmo NON e` adatto per la realizzazione della visita in
      profondita`.
}


    procedure VisitaInAmpiezza (var grafo: TipoGrafo; nodo_iniz: TipoNodo);
    { Effettua una visita del grafo in ampiezza a partire da nodo_iniz.
      Utilizza una pila. }
    var
      i,j              : TipoNodo;
      nodi_da_visitare : TipoCoda;
      nodi_marcati     : array [TipoNodo] of boolean;
    
    begin
      { inizializzazione }
{ 1 } for i := 1 to NumNodi do
        nodi_marcati[i] := FALSE;
    
{ 2 } InitCoda(nodi_da_visitare);
{ 3 } nodi_marcati[nodo_iniz] := TRUE;
{ 4 } InCoda(nodi_da_visitare, nodo_iniz);
    
      { cicla finche` ci sono nodi da visitare }
{ 5 } while not TestCodaVuota(nodi_da_visitare) do
      begin
        { prendi il prossimo nodo i da visitare e analizzalo }
{ 6 }   OutCoda(nodi_da_visitare, i);
{ 7 }   Analizza(i);

        { marca i successori non visitati del nodo i e aggiungili ai nodi da
          visitare }
{ 8 }   for j := 1 to NumNodi do
          if TestEsisteArco(grafo, i, j) and (not nodi_marcati[j]) then
          begin
{ 9 }       nodi_marcati[j] := TRUE;
{10 }       InCoda(nodi_da_visitare, j)
          end
      end
    end; { VisitaInAmpiezza }
    

procedure VisitaRicInProfondita (var grafo: TipoGrafo; nodo_iniz: TipoNodo);
{ Effettua una visita del grafo in profondita` a partire da nodo_iniz.
  Utilizza una procedura ricorsiva. }
var
  i            : TipoNodo;
  nodi_marcati : array [TipoNodo] of boolean;

  procedure VisitaRicorsiva (var grafo: TipoGrafo; i: TipoNodo);
  { Procedura ricorsiva che effettua la visita in profondita`. }
  var
    j : TipoNodo;
{ var globale
    nodi_marcati : array [TipoNodo] of boolean }

  begin
    { analizza il nodo i }
    Analizza(i);

    { marca i successori non visitati del nodo i e li visita }
    for j := 1 to NumNodi do
      if TestEsisteArco(grafo, i, j) and (not nodi_marcati[j]) then
      begin
        {N.B. e` fondamentale PRIMA marcare il nodo e POI visitarlo }
        nodi_marcati[j] := TRUE;
        VisitaRicorsiva(grafo, j)
      end
  end; { VisitaRicorsiva }

begin { VisitaRicInProfondita }
  { inizializzazione }
  for i := 1 to NumNodi do
    nodi_marcati[i] := FALSE;
    
  { marca il nodo iniziale inizia la visita ricorsiva }
  nodi_marcati[nodo_iniz] := TRUE;
  VisitaRicorsiva(grafo, nodo_iniz)
end; { VisitaRicInProfondita }
